@@ -1,4 +1,4 @@
-/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
+// -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
 
 const St = imports.gi.St;
 const Shell = imports.gi.Shell;
@@ -29,8 +29,8 @@ DocInfo.prototype = {
         return St.TextureCache.get_default().load_recent_thumbnail(size, this.recentInfo);
     },
 
-    launch : function() {
-        Shell.DocSystem.get_default().open(this.recentInfo);
+    launch : function(workspaceIndex) {
+        Shell.DocSystem.get_default().open(this.recentInfo, workspaceIndex);
     },
 
     matchTerms: function(terms) {
