@@ -1,4 +1,4 @@
-/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
+// -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
 
 const Clutter = imports.gi.Clutter;
 const Lang = imports.lang;
@@ -46,6 +46,7 @@ BoxPointer.prototype = {
         this._yOffset = 0;
         this._xPosition = 0;
         this._yPosition = 0;
+        this._sourceAlignment = 0.5;
     },
 
     show: function(animate, onComplete) {
@@ -75,7 +76,7 @@ BoxPointer.prototype = {
         Tweener.addTween(this, { opacity: 255,
                                  xOffset: 0,
                                  yOffset: 0,
-                                 transition: "linear",
+                                 transition: 'linear',
                                  onComplete: onComplete,
                                  time: POPUP_ANIMATION_TIME });
     },
@@ -106,7 +107,7 @@ BoxPointer.prototype = {
         Tweener.addTween(this, { opacity: 0,
                                  xOffset: xOffset,
                                  yOffset: yOffset,
-                                 transition: "linear",
+                                 transition: 'linear',
                                  time: POPUP_ANIMATION_TIME,
                                  onComplete: Lang.bind(this, function () {
                                      this.actor.hide();
@@ -180,7 +181,7 @@ BoxPointer.prototype = {
         this.bin.allocate(childBox, flags);
 
         if (this._sourceActor && this._sourceActor.mapped)
-            this._reposition(this._sourceActor, this._alignment);
+            this._reposition(this._sourceActor, this._arrowAlignment);
     },
 
     _drawBorder: function(area) {
@@ -312,24 +313,37 @@ BoxPointer.prototype = {
         this.actor.show();
 
         this._sourceActor = sourceActor;
-        this._alignment = alignment;
+        this._arrowAlignment = alignment;
 
         this._reposition(sourceActor, alignment);
     },
 
+    setSourceAlignment: function(alignment) {
+        this._sourceAlignment = alignment;
+
+        if (!this._sourceActor)
+            return;
+
+        // We need to show it now to force an allocation,
+        // so that we can query the correct size.
+        this.actor.show();
+
+        this._reposition(this._sourceActor, this._arrowAlignment);
+    },
+
     _reposition: function(sourceActor, alignment) {
         // Position correctly relative to the sourceActor
         let sourceNode = sourceActor.get_theme_node();
         let sourceContentBox = sourceNode.get_content_box(sourceActor.get_allocation_box());
         let sourceAllocation = Shell.util_get_transformed_allocation(sourceActor);
-        let sourceCenterX = sourceAllocation.x1 + sourceContentBox.x1 + (sourceContentBox.x2 - sourceContentBox.x1) / 2;
-        let sourceCenterY = sourceAllocation.y1 + sourceContentBox.y1 + (sourceContentBox.y2 - sourceContentBox.y1) / 2;
+        let sourceCenterX = sourceAllocation.x1 + sourceContentBox.x1 + (sourceContentBox.x2 - sourceContentBox.x1) * this._sourceAlignment;
+        let sourceCenterY = sourceAllocation.y1 + sourceContentBox.y1 + (sourceContentBox.y2 - sourceContentBox.y1) * this._sourceAlignment;
         let [minWidth, minHeight, natWidth, natHeight] = this.actor.get_preferred_size();
 
         // We also want to keep it onscreen, and separated from the
         // edge by the same distance as the main part of the box is
         // separated from its sourceActor
-        let primary = Main.layoutManager.primaryMonitor;
+        let monitor = Main.layoutManager.findMonitorForActor(sourceActor);
         let themeNode = this.actor.get_theme_node();
         let borderWidth = themeNode.get_length('-arrow-border-width');
         let arrowBase = themeNode.get_length('-arrow-base');
@@ -364,8 +378,8 @@ BoxPointer.prototype = {
         case St.Side.BOTTOM:
             resX = sourceCenterX - (halfMargin + (natWidth - margin) * alignment);
 
-            resX = Math.max(resX, primary.x + 10);
-            resX = Math.min(resX, primary.x + primary.width - (10 + natWidth));
+            resX = Math.max(resX, monitor.x + 10);
+            resX = Math.min(resX, monitor.x + monitor.width - (10 + natWidth));
             this.setArrowOrigin(sourceCenterX - resX);
             break;
 
@@ -373,8 +387,8 @@ BoxPointer.prototype = {
         case St.Side.RIGHT:
             resY = sourceCenterY - (halfMargin + (natHeight - margin) * alignment);
 
-            resY = Math.max(resY, primary.y + 10);
-            resY = Math.min(resY, primary.y + primary.height - (10 + natHeight));
+            resY = Math.max(resY, monitor.y + 10);
+            resY = Math.min(resY, monitor.y + monitor.height - (10 + natHeight));
 
             this.setArrowOrigin(sourceCenterY - resY);
             break;
