@@ -1,4 +1,4 @@
-/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
+// -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
 
 const Clutter = imports.gi.Clutter;
 const Gdk = imports.gi.Gdk;
@@ -94,7 +94,7 @@ CtrlAltTabManager.prototype = {
         return a.x - b.x;
     },
 
-    popup: function(backwards) {
+    popup: function(backwards, mask) {
         // Start with the set of focus groups that are currently mapped
         let items = this._items.filter(function (item) { return item.proxy.mapped; });
 
@@ -123,7 +123,16 @@ CtrlAltTabManager.prototype = {
             return;
 
         items.sort(Lang.bind(this, this._sortItems));
-        new CtrlAltTabPopup().show(items, backwards);
+
+        if (!this._popup) {
+            this._popup = new CtrlAltTabPopup();
+            this._popup.show(items, backwards, mask);
+
+            this._popup.actor.connect('destroy',
+                                      Lang.bind(this, function() {
+                                          this._popup = null;
+                                      }));
+        }
     }
 };
 
@@ -147,20 +156,21 @@ CtrlAltTabPopup.prototype = {
         this.actor.connect('destroy', Lang.bind(this, this._onDestroy));
 
         this._haveModal = false;
+        this._modifierMask = 0;
         this._selection = 0;
 
         Main.uiGroup.add_actor(this.actor);
     },
 
     _getPreferredWidth: function (actor, forHeight, alloc) {
-        let primary = global.get_primary_monitor();
+        let primary = Main.layoutManager.primaryMonitor;
 
         alloc.min_size = primary.width;
         alloc.natural_size = primary.width;
     },
 
     _getPreferredHeight: function (actor, forWidth, alloc) {
-        let primary = global.get_primary_monitor();
+        let primary = Main.layoutManager.primaryMonitor;
 
         alloc.min_size = primary.height;
         alloc.natural_size = primary.height;
@@ -168,7 +178,7 @@ CtrlAltTabPopup.prototype = {
 
     _allocate: function (actor, box, flags) {
         let childBox = new Clutter.ActorBox();
-        let primary = global.get_primary_monitor();
+        let primary = Main.layoutManager.primaryMonitor;
 
         let leftPadding = this.actor.get_theme_node().get_padding(St.Side.LEFT);
         let vPadding = this.actor.get_theme_node().get_vertical_padding();
@@ -183,10 +193,11 @@ CtrlAltTabPopup.prototype = {
         this._switcher.actor.allocate(childBox, flags);
     },
 
-    show : function(items, startBackwards) {
+    show : function(items, startBackwards, mask) {
         if (!Main.pushModal(this.actor))
             return false;
         this._haveModal = true;
+        this._modifierMask = AltTab.primaryModifier(mask);
 
         this._keyPressEventId = this.actor.connect('key-press-event', Lang.bind(this, this._keyPressEvent));
         this._keyReleaseEventId = this.actor.connect('key-release-event', Lang.bind(this, this._keyReleaseEvent));
@@ -200,7 +211,7 @@ CtrlAltTabPopup.prototype = {
         this._select(this._selection);
 
         let [x, y, mods] = global.get_pointer();
-        if (!(mods & Gdk.ModifierType.MOD1_MASK)) {
+        if (!(mods & this._modifierMask)) {
             this._finish();
             return false;
         }
@@ -246,7 +257,7 @@ CtrlAltTabPopup.prototype = {
 
     _keyReleaseEvent : function(actor, event) {
         let [x, y, mods] = global.get_pointer();
-        let state = mods & Clutter.ModifierType.MOD1_MASK;
+        let state = mods & this._modifierMask;
 
         if (state == 0)
             this._finish();
