@@ -1,4 +1,4 @@
-/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
+// -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
 
 const Caribou = imports.gi.Caribou;
 const Clutter = imports.gi.Clutter;
@@ -15,9 +15,11 @@ const Main = imports.ui.main;
 const MessageTray = imports.ui.messageTray;
 
 const KEYBOARD_SCHEMA = 'org.gnome.shell.keyboard';
-const SHOW_KEYBOARD = 'show-keyboard';
 const KEYBOARD_TYPE = 'keyboard-type';
 
+const A11Y_APPLICATIONS_SCHEMA = 'org.gnome.desktop.a11y.applications';
+const SHOW_KEYBOARD = 'screen-keyboard-enabled';
+
 // Key constants taken from Antler
 // FIXME: ought to be moved into libcaribou
 const PRETTY_KEYS = {
@@ -73,7 +75,7 @@ Key.prototype = {
         this._extended_keys = this._key.get_extended_keys();
         this._extended_keyboard = null;
 
-        if (this._key.name == "Control_L" || this._key.name == "Alt_L")
+        if (this._key.name == 'Control_L' || this._key.name == 'Alt_L')
             this._key.latch = true;
 
         this._key.connect('key-pressed', Lang.bind(this, function ()
@@ -207,6 +209,8 @@ Keyboard.prototype = {
 
         this._keyboardSettings = new Gio.Settings({ schema: KEYBOARD_SCHEMA });
         this._keyboardSettings.connect('changed', Lang.bind(this, this._settingsChanged));
+        this._a11yApplicationsSettings = new Gio.Settings({ schema: A11Y_APPLICATIONS_SCHEMA });
+        this._a11yApplicationsSettings.connect('changed', Lang.bind(this, this._settingsChanged));
         this._settingsChanged();
     },
 
@@ -214,8 +218,8 @@ Keyboard.prototype = {
         this._redraw();
     },
 
-    _settingsChanged: function () {
-        this._enableKeyboard = this._keyboardSettings.get_boolean(SHOW_KEYBOARD);
+    _settingsChanged: function (settings, key) {
+        this._enableKeyboard = this._a11yApplicationsSettings.get_boolean(SHOW_KEYBOARD);
         if (!this._enableKeyboard && !this._keyboard)
             return;
         if (this._enableKeyboard && this._keyboard &&
@@ -224,9 +228,20 @@ Keyboard.prototype = {
 
         if (this._keyboard)
             this._destroyKeyboard();
-        if (this._enableKeyboard)
-            this._setupKeyboard();
-        else
+
+        if (this._enableKeyboard) {
+            // If we've been called because the setting actually just
+            // changed to true (as opposed to being called from
+            // this._init()), then we want to pop up the keyboard.
+            let showKeyboard = (settings != null);
+
+            // However, caribou-gtk-module or this._onKeyFocusChanged
+            // will probably immediately tell us to hide it, so we
+            // have to fake things out so we'll ignore that request.
+            if (showKeyboard)
+                this._timestamp = global.display.get_current_time_roundtrip() + 1;
+            this._setupKeyboard(showKeyboard);
+        } else
             Main.layoutManager.hideKeyboard(true);
     },
 
@@ -242,7 +257,7 @@ Keyboard.prototype = {
         this._destroySource();
     },
 
-    _setupKeyboard: function() {
+    _setupKeyboard: function(show) {
         this.actor = new St.BoxLayout({ name: 'keyboard', vertical: true, reactive: true });
         Main.layoutManager.keyboardBox.add_actor(this.actor);
         Main.layoutManager.trackChrome(this.actor);
@@ -259,20 +274,26 @@ Keyboard.prototype = {
 
         this._keyboardNotifyId = this._keyboard.connect('notify::active-group', Lang.bind(this, this._onGroupChanged));
         this._focusNotifyId = global.stage.connect('notify::key-focus', Lang.bind(this, this._onKeyFocusChanged));
-        this._createSource();
+
+        if (show)
+            this.show();
+        else
+            this._createSource();
     },
 
     _onKeyFocusChanged: function () {
         let focus = global.stage.key_focus;
 
-        // Showing an extended key popup will grab focus, but ignore that
-        if (focus && focus._extended_keys)
+        // Showing an extended key popup and clicking a key from the extended keys
+        // will grab focus, but ignore that
+        if (focus && (focus._extended_keys || (focus._key && focus._key.extended_key)))
             return;
 
+        let time = global.current_event_time();
         if (focus instanceof Clutter.Text)
-            this.show();
+            this.Show(time);
         else
-            this.hide();
+            this.Hide(time);
     },
 
     _addKeys: function () {
@@ -301,7 +322,8 @@ Keyboard.prototype = {
     },
 
     _getTrayIcon: function () {
-        let trayButton = new St.Button ({ label: "tray", style_class: 'keyboard-key' });
+        let trayButton = new St.Button ({ label: _("tray"),
+                                          style_class: 'keyboard-key' });
         trayButton.key_width = 1;
         trayButton.connect('button-press-event', Lang.bind(this, function () {
             Main.messageTray.toggle();
@@ -335,7 +357,7 @@ Keyboard.prototype = {
                     right_box.add(button.actor);
                 else
                     left_box.add(button.actor);
-                if (key.name == "Caribou_Prefs") {
+                if (key.name == 'Caribou_Prefs') {
                     key.connect('key-released', Lang.bind(this, this.hide));
 
                     // Add new key for hiding message tray
@@ -472,6 +494,9 @@ Keyboard.prototype = {
 
     // D-Bus methods
     Show: function(timestamp) {
+        if (!this._enableKeyboard)
+            return;
+
         if (timestamp - this._timestamp < 0)
             return;
 
@@ -480,6 +505,9 @@ Keyboard.prototype = {
     },
 
     Hide: function(timestamp) {
+        if (!this._enableKeyboard)
+            return;
+
         if (timestamp - this._timestamp < 0)
             return;
 
@@ -488,11 +516,17 @@ Keyboard.prototype = {
     },
 
     SetCursorLocation: function(x, y, w, h) {
-        this._setLocation(x, y);
+        if (!this._enableKeyboard)
+            return;
+
+//        this._setLocation(x, y);
     },
 
     SetEntryLocation: function(x, y, w, h) {
-        this._setLocation(x, y);
+        if (!this._enableKeyboard)
+            return;
+
+//        this._setLocation(x, y);
     },
 
     get Name() {
