@@ -1,4 +1,4 @@
-/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
+// -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
 
 const Clutter = imports.gi.Clutter;
 const Lang = imports.lang;
@@ -216,10 +216,10 @@ LayoutManager.prototype = {
         let monitorLeft = monitor.x, monitorRight = monitor.x + monitor.width;
         let primaryLeft = primary.x, primaryRight = primary.x + primary.width;
 
-        if ((monitorLeft >= primaryLeft && monitorLeft <= primaryRight) ||
-            (monitorRight >= primaryLeft && monitorRight <= primaryRight) ||
-            (primaryLeft >= monitorLeft && primaryLeft <= monitorRight) ||
-            (primaryRight >= monitorLeft && primaryRight <= monitorRight))
+        if ((monitorLeft >= primaryLeft && monitorLeft < primaryRight) ||
+            (monitorRight > primaryLeft && monitorRight <= primaryRight) ||
+            (primaryLeft >= monitorLeft && primaryLeft < monitorRight) ||
+            (primaryRight > monitorLeft && primaryRight <= monitorRight))
             return true;
 
         return false;
@@ -369,6 +369,10 @@ LayoutManager.prototype = {
     // Removes @actor from the chrome
     removeChrome: function(actor) {
         this._chrome.removeActor(actor);
+    },
+
+    findMonitorForActor: function(actor) {
+        return this._chrome.findMonitorForActor(actor);
     }
 };
 Signals.addSignalMethods(LayoutManager.prototype);
@@ -621,7 +625,7 @@ Chrome.prototype = {
         // We can't use Params.parse here because we want to drop
         // the extra values like ancestorData.actor
         for (let prop in defaultParams) {
-            if (!params[prop])
+            if (!params.hasOwnProperty(prop))
                 params[prop] = ancestorData[prop];
         }
 
@@ -700,7 +704,7 @@ Chrome.prototype = {
             else if (this._inOverview)
                 visible = true;
             else if (!actorData.visibleInFullscreen &&
-                     this._findMonitorForActor(actorData.actor).inFullscreen)
+                     this.findMonitorForActor(actorData.actor).inFullscreen)
                 visible = false;
             else
                 visible = true;
@@ -762,7 +766,7 @@ Chrome.prototype = {
 
     // This call guarantees that we return some monitor to simplify usage of it
     // In practice all tracked actors should be visible on some monitor anyway
-    _findMonitorForActor: function(actor) {
+    findMonitorForActor: function(actor) {
         let [x, y] = actor.get_transformed_position();
         let [w, h] = actor.get_transformed_size();
         let monitor = this._findMonitorForRect(x, y, w, h);
@@ -821,6 +825,18 @@ Chrome.prototype = {
                     monitor.inFullscreen = true;
             }
             if (layer == Meta.StackLayer.OVERRIDE_REDIRECT) {
+                // Check whether the window is screen sized
+                let isScreenSized =
+                    (window.x == 0 && window.y == 0 &&
+                    window.width == global.screen_width &&
+                    window.height == global.screen_height);
+
+                if (isScreenSized) {
+                    for (let i = 0; i < this._monitors.length; i++)
+                        this._monitors[i].inFullscreen = true;
+                }
+
+                // Or whether it is monitor sized
                 let monitor = this._findMonitorForWindow(window);
                 if (monitor &&
                     window.x <= monitor.x &&
