@@ -1,4 +1,4 @@
-/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
+// -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
 
 const Clutter = imports.gi.Clutter;
 const Cogl = imports.gi.Cogl;
@@ -6,6 +6,7 @@ const GConf = imports.gi.GConf;
 const GLib = imports.gi.GLib;
 const Gio = imports.gi.Gio;
 const Gtk = imports.gi.Gtk;
+const Meta = imports.gi.Meta;
 const Pango = imports.gi.Pango;
 const St = imports.gi.St;
 const Shell = imports.gi.Shell;
@@ -16,6 +17,7 @@ const Mainloop = imports.mainloop;
 const History = imports.misc.history;
 const ExtensionSystem = imports.ui.extensionSystem;
 const Link = imports.ui.link;
+const ShellEntry = imports.ui.shellEntry;
 const Tweener = imports.ui.tweener;
 const Main = imports.ui.main;
 
@@ -156,9 +158,9 @@ Signals.addSignalMethods(Notebook.prototype);
 function objectToString(o) {
     if (typeof(o) == typeof(objectToString)) {
         // special case this since the default is way, way too verbose
-        return "<js function>";
+        return '<js function>';
     } else {
-        return "" + o;
+        return '' + o;
     }
 }
 
@@ -321,7 +323,7 @@ ObjInspector.prototype = {
                     link = new St.Label({ text: '<error>' });
                 }
                 let hbox = new St.BoxLayout();
-                let propText = propName + ": " + valueStr;
+                let propText = propName + ': ' + valueStr;
                 hbox.add(new St.Label({ text: propName + ': ' }));
                 hbox.add(link);
                 this._container.add_actor(hbox);
@@ -342,7 +344,7 @@ ObjInspector.prototype = {
             this.actor.move_anchor_point(Math.floor(sourceX + sourceWidth / 2),
                                          Math.floor(sourceY + sourceHeight / 2));
             Tweener.addTween(this.actor, { scale_x: 1, scale_y: 1,
-                                           transition: "easeOutQuad",
+                                           transition: 'easeOutQuad',
                                            time: 0.2 });
         } else {
             this.actor.set_scale(1, 1);
@@ -606,6 +608,9 @@ Memory.prototype = {
         this._gjs_closure = new St.Label();
         this.actor.add(this._gjs_closure);
 
+        this._last_gc_seconds_ago = new St.Label();
+        this.actor.add(this._last_gc_seconds_ago);
+
         this._gcbutton = new St.Button({ label: 'Full GC',
                                          style_class: 'lg-obj-inspector-button' });
         this._gcbutton.connect('clicked', Lang.bind(this, function () { global.gc(); this._renderText(); }));
@@ -625,6 +630,7 @@ Memory.prototype = {
         this._gjs_gobject.text = 'gjs_gobject: ' + memInfo.gjs_gobject;
         this._gjs_function.text = 'gjs_function: ' + memInfo.gjs_function;
         this._gjs_closure.text = 'gjs_closure: ' + memInfo.gjs_closure;
+        this._last_gc_seconds_ago.text = 'last_gc_seconds_ago: ' + memInfo.last_gc_seconds_ago;
     }
 };
 
@@ -705,27 +711,23 @@ Extensions.prototype = {
                                          text: meta.description || 'No description' });
         box.add(description, { expand: true });
 
-        let metaBox = new St.BoxLayout();
+        let metaBox = new St.BoxLayout({ style_class: 'lg-extension-meta' });
         box.add(metaBox);
         let stateString = this._stateToString(meta.state);
         let state = new St.Label({ style_class: 'lg-extension-state',
                                    text: this._stateToString(meta.state) });
-
-        let actionsContainer = new St.Bin({ x_align: St.Align.END });
-        metaBox.add(actionsContainer);
-        let actionsBox = new St.BoxLayout({ style_class: 'lg-extension-actions' });
-        actionsContainer.set_child(actionsBox);
+        metaBox.add(state);
 
         let viewsource = new Link.Link({ label: _("View Source") });
         viewsource.actor._extensionMeta = meta;
         viewsource.actor.connect('clicked', Lang.bind(this, this._onViewSource));
-        actionsBox.add(viewsource.actor);
+        metaBox.add(viewsource.actor);
 
         if (meta.url) {
             let webpage = new Link.Link({ label: _("Web Page") });
             webpage.actor._extensionMeta = meta;
             webpage.actor.connect('clicked', Lang.bind(this, this._onWebPage));
-            actionsBox.add(webpage.actor);
+            metaBox.add(webpage.actor);
         }
 
         return box;
@@ -761,9 +763,13 @@ LookingGlass.prototype = {
                                         Lang.bind(this, this._updateFont));
         this._updateFont();
 
-        // we add it to the chrome because we want it to appear to slide
-        // out from underneath the panel
-        Main.layoutManager.addChrome(this.actor);
+        // We want it to appear to slide out from underneath the panel
+        Main.layoutManager.panelBox.add_actor(this.actor);
+        this.actor.lower_bottom();
+        Main.layoutManager.panelBox.connect('allocation-changed',
+                                            Lang.bind(this, this._queueResize));
+        Main.layoutManager.keyboardBox.connect('allocation-changed',
+                                               Lang.bind(this, this._queueResize));
 
         this._objInspector = new ObjInspector();
         Main.uiGroup.add_actor(this._objInspector.actor);
@@ -811,6 +817,7 @@ LookingGlass.prototype = {
         entryArea.add(label);
 
         this._entry = new St.Entry({ can_focus: true });
+        ShellEntry.addContextMenu(this._entry);
         entryArea.add(this._entry, { expand: true });
 
         this._windowList = new WindowList();
@@ -844,6 +851,8 @@ LookingGlass.prototype = {
 
         this._history = new History.HistoryManager({ gsettingsKey: HISTORY_KEY, 
                                                      entry: this._entry.clutter_text });
+
+        this._resize();
     },
 
     _updateFont: function() {
@@ -918,13 +927,18 @@ LookingGlass.prototype = {
             this.open();
     },
 
-    _resizeTo: function(actor) {
+    _queueResize: function() {
+        Meta.later_add(Meta.LaterType.BEFORE_REDRAW,
+                       Lang.bind(this, function () { this._resize(); }));
+    },
+
+    _resize: function() {
         let primary = Main.layoutManager.primaryMonitor;
         let myWidth = primary.width * 0.7;
-        let myHeight = primary.height * 0.7;
-        let [srcX, srcY] = actor.get_transformed_position();
-        this.actor.x = srcX + (primary.width - myWidth) / 2;
-        this._hiddenY = srcY + actor.height - myHeight - 4; // -4 to hide the top corners
+        let availableHeight = primary.height - Main.layoutManager.keyboardBox.height;
+        let myHeight = Math.min(primary.height * 0.7, availableHeight * 0.9);
+        this.actor.x = (primary.width - myWidth) / 2;
+        this._hiddenY = this.actor.get_parent().height - myHeight - 4; // -4 to hide the top corners
         this._targetY = this._hiddenY + myHeight;
         this.actor.y = this._hiddenY;
         this.actor.width = myWidth;
@@ -934,14 +948,6 @@ LookingGlass.prototype = {
                                               this._targetY + Math.floor(myHeight * 0.1));
     },
 
-    slaveTo: function(actor) {
-        this._slaveTo = actor;
-        actor.connect('notify::allocation', Lang.bind(this, function () {
-            this._resizeTo(actor);
-        }));
-        this._resizeTo(actor);
-    },
-
     insertObject: function(obj) {
         this._pushResult('<insert>', obj);
     },
@@ -974,7 +980,6 @@ LookingGlass.prototype = {
 
         this._notebook.selectIndex(0);
         this.actor.show();
-        this.actor.lower_bottom();
         this._open = true;
         this._history.lastItem();
 
@@ -1005,7 +1010,6 @@ LookingGlass.prototype = {
 
         Main.popModal(this._entry);
 
-        this.actor.lower_bottom();
         Tweener.addTween(this.actor, { time: 0.5 / St.get_slow_down_factor(),
                                        transition: 'easeOutQuad',
                                        y: this._hiddenY,
