@@ -1,4 +1,4 @@
-/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
+// -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
 
 const DBus = imports.dbus;
 const Gio = imports.gi.Gio;
@@ -257,7 +257,7 @@ Client.prototype = {
         // FIXME: We don't have a 'chat room' icon (bgo #653737) use
         // system-users for now as Empathy does.
         let source = new ApproverSource(dispatchOp, _("Invitation"),
-                                        Shell.util_icon_from_string('system-users'));
+                                        Gio.icon_new_for_string('system-users'));
         Main.messageTray.add(source);
 
         let notif = new RoomInviteNotification(source, dispatchOp, channel, contacts[0]);
@@ -325,8 +325,8 @@ Client.prototype = {
 
         // We got the TpContact
         let source = new ApproverSource(dispatchOp, _("Call"), isVideo ?
-                                        Shell.util_icon_from_string('camera-web') :
-                                        Shell.util_icon_from_string('audio-input-microphone'));
+                                        Gio.icon_new_for_string('camera-web') :
+                                        Gio.icon_new_for_string('audio-input-microphone'));
         Main.messageTray.add(source);
 
         let notif = new AudioVideoNotification(source, dispatchOp, channel, contacts[0], isVideo);
@@ -539,9 +539,13 @@ ChatSource.prototype = {
 
     _updateAlias: function() {
         let oldAlias = this.title;
-        this.setTitle(this._contact.get_alias());
-        this._notification.appendAliasChange(oldAlias, this.title);
-        this.pushNotification(this._notification);
+        let newAlias = this._contact.get_alias();
+
+        if (oldAlias == newAlias)
+            return;
+
+        this.setTitle(newAlias);
+        this._notification.appendAliasChange(oldAlias, newAlias);
     },
 
     createNotificationIcon: function() {
@@ -570,7 +574,7 @@ ChatSource.prototype = {
     open: function(notification) {
           if (this._client.is_handling_channel(this._channel)) {
               // We are handling the channel, try to pass it to Empathy
-              this._client.delegate_channels_async([this._channel], global.get_current_time(), "", null);
+              this._client.delegate_channels_async([this._channel], global.get_current_time(), '', null);
           }
           else {
               // We are not the handler, just ask to present the channel
@@ -1198,6 +1202,7 @@ AudioVideoNotification.prototype = {
         this.setResident(true);
 
         this.addButton('reject', _("Reject"));
+        /* translators: this is a button label (verb), not a noun */
         this.addButton('answer', _("Answer"));
 
         this.connect('action-invoked', Lang.bind(this, function(self, action) {
