@@ -24,22 +24,25 @@
  * IN THE SOFTWARE.
  */
 
-#include <config.h>
-#include <gtk/gtk.h>
-#include <gdk/gdkx.h>
-#include <clutter/x11/clutter-x11.h>
-#include <string.h>
-#include <stdlib.h>
+#include "config.h"
+
 #include <locale.h>
+#include <stdlib.h>
+#include <string.h>
 
+#include <clutter/x11/clutter-x11.h>
+#include <gdk/gdkx.h>
+#include <girepository.h>
 #include <gjs/gjs.h>
+#include <gtk/gtk.h>
+
+#include "shell-global.h"
+#include "shell-global-private.h"
 
-static char **include_path = NULL;
 static char *command = NULL;
 
 static GOptionEntry entries[] = {
   { "command", 'c', 0, G_OPTION_ARG_STRING, &command, "Program passed in as a string", "COMMAND" },
-  { "include-path", 'I', 0, G_OPTION_ARG_STRING_ARRAY, &include_path, "Add the directory DIR to the list of directories to search for js files.", "DIR" },
   { NULL }
 };
 
@@ -59,10 +62,10 @@ event_filter (GdkXEvent *xevent,
 int
 main(int argc, char **argv)
 {
-  char *command_line;
   GOptionContext *context;
   ClutterActor *stage;
   GError *error = NULL;
+  ShellGlobal *global;
   GjsContext *js_context;
   char *script;
   const char *filename;
@@ -94,12 +97,9 @@ main(int argc, char **argv)
   setlocale (LC_ALL, "");
   g_type_init ();
 
-  command_line = g_strjoinv (" ", argv);
-  g_debug ("Command line: %s", command_line);
-  g_free (command_line);
-
-  g_debug ("Creating new context to eval console script");
-  js_context = gjs_context_new_with_search_path (include_path);
+  _shell_global_init (NULL);
+  global = shell_global_get ();
+  js_context = _shell_global_get_gjs_context (global);
 
   /* prepare command line arguments */
   if (!gjs_context_define_string_array (js_context, "ARGV",
@@ -131,6 +131,14 @@ main(int argc, char **argv)
   clutter_stage_set_title (CLUTTER_STAGE (stage), title);
   g_free (title);
 
+#if HAVE_BLUETOOTH
+  /* The module imports are all so intertwined that if the test
+   * imports anything in js/ui, it will probably eventually end up
+   * pulling in ui/status/bluetooth.js. So we need this.
+   */
+  g_irepository_prepend_search_path (BLUETOOTH_DIR);
+#endif
+
   /* evaluate the script */
   error = NULL;
   if (!gjs_context_eval (js_context, script, len,
