@@ -208,25 +208,26 @@ _shell_marshal_VOID__OBJECT_OBJECT (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,OBJECT,BOOLEAN (./shell-marshal.list:6) */
+/* VOID:STRING,UINT,OBJECT,BOOLEAN (./shell-marshal.list:6) */
 void
-_shell_marshal_VOID__STRING_OBJECT_BOOLEAN (GClosure     *closure,
-                                            GValue       *return_value G_GNUC_UNUSED,
-                                            guint         n_param_values,
-                                            const GValue *param_values,
-                                            gpointer      invocation_hint G_GNUC_UNUSED,
-                                            gpointer      marshal_data)
+_shell_marshal_VOID__STRING_UINT_OBJECT_BOOLEAN (GClosure     *closure,
+                                                 GValue       *return_value G_GNUC_UNUSED,
+                                                 guint         n_param_values,
+                                                 const GValue *param_values,
+                                                 gpointer      invocation_hint G_GNUC_UNUSED,
+                                                 gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__STRING_OBJECT_BOOLEAN) (gpointer     data1,
-                                                            gpointer     arg_1,
-                                                            gpointer     arg_2,
-                                                            gboolean     arg_3,
-                                                            gpointer     data2);
-  register GMarshalFunc_VOID__STRING_OBJECT_BOOLEAN callback;
+  typedef void (*GMarshalFunc_VOID__STRING_UINT_OBJECT_BOOLEAN) (gpointer     data1,
+                                                                 gpointer     arg_1,
+                                                                 guint        arg_2,
+                                                                 gpointer     arg_3,
+                                                                 gboolean     arg_4,
+                                                                 gpointer     data2);
+  register GMarshalFunc_VOID__STRING_UINT_OBJECT_BOOLEAN callback;
   register GCClosure *cc = (GCClosure*) closure;
   register gpointer data1, data2;
 
-  g_return_if_fail (n_param_values == 4);
+  g_return_if_fail (n_param_values == 5);
 
   if (G_CCLOSURE_SWAP_DATA (closure))
     {
@@ -238,12 +239,13 @@ _shell_marshal_VOID__STRING_OBJECT_BOOLEAN (GClosure     *closure,
       data1 = g_value_peek_pointer (param_values + 0);
       data2 = closure->data;
     }
-  callback = (GMarshalFunc_VOID__STRING_OBJECT_BOOLEAN) (marshal_data ? marshal_data : cc->callback);
+  callback = (GMarshalFunc_VOID__STRING_UINT_OBJECT_BOOLEAN) (marshal_data ? marshal_data : cc->callback);
 
   callback (data1,
             g_marshal_value_peek_string (param_values + 1),
-            g_marshal_value_peek_object (param_values + 2),
-            g_marshal_value_peek_boolean (param_values + 3),
+            g_marshal_value_peek_uint (param_values + 2),
+            g_marshal_value_peek_object (param_values + 3),
+            g_marshal_value_peek_boolean (param_values + 4),
             data2);
 }
 
@@ -327,3 +329,44 @@ _shell_marshal_VOID__STRING_STRING_STRING_STRING_BOXED (GClosure     *closure,
             data2);
 }
 
+/* VOID:STRING,OBJECT,STRING,BOXED (./shell-marshal.list:9) */
+void
+_shell_marshal_VOID__STRING_OBJECT_STRING_BOXED (GClosure     *closure,
+                                                 GValue       *return_value G_GNUC_UNUSED,
+                                                 guint         n_param_values,
+                                                 const GValue *param_values,
+                                                 gpointer      invocation_hint G_GNUC_UNUSED,
+                                                 gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__STRING_OBJECT_STRING_BOXED) (gpointer     data1,
+                                                                 gpointer     arg_1,
+                                                                 gpointer     arg_2,
+                                                                 gpointer     arg_3,
+                                                                 gpointer     arg_4,
+                                                                 gpointer     data2);
+  register GMarshalFunc_VOID__STRING_OBJECT_STRING_BOXED callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+
+  g_return_if_fail (n_param_values == 5);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__STRING_OBJECT_STRING_BOXED) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_marshal_value_peek_string (param_values + 1),
+            g_marshal_value_peek_object (param_values + 2),
+            g_marshal_value_peek_string (param_values + 3),
+            g_marshal_value_peek_boxed (param_values + 4),
+            data2);
+}
+
