@@ -1,6 +1,8 @@
 /* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /*
- * st-im-text.c
+ * st-im-text.c: Text widget with input method support
+ *
+ * Copyright 2009 Red Hat, Inc.
  *
  * This started as a copy of ClutterIMText converted to use
  * GtkIMContext rather than ClutterIMContext. Original code:
@@ -18,10 +20,8 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
- * 02110-1301 USA
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
 /**
@@ -61,6 +61,7 @@
 #include <glib.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
+#include <gdk/gdkx.h>
 #include <X11/extensions/XKB.h>
 
 #include "st-im-text.h"
@@ -102,7 +103,6 @@ update_im_cursor_location (StIMText *self)
 {
   StIMTextPrivate *priv = self->priv;
   ClutterText *clutter_text = CLUTTER_TEXT (self);
-  ClutterActor *parent;
   gint position;
   gfloat cursor_x, cursor_y, cursor_height;
   gfloat actor_x, actor_y;
@@ -112,25 +112,7 @@ update_im_cursor_location (StIMText *self)
   clutter_text_position_to_coords (clutter_text, position,
                                    &cursor_x, &cursor_y, &cursor_height);
 
-  /* This is a workaround for a bug in Clutter where
-   * clutter_actor_get_transformed_position doesn't work during
-   * clutter_actor_paint() because the actor has already set up
-   * a model-view matrix.
-   *
-   * http://bugzilla.openedhand.com/show_bug.cgi?id=1115
-   */
-  actor_x = actor_y = 0.;
-  parent = CLUTTER_ACTOR (self);
-  while (parent)
-    {
-      gfloat x, y;
-
-      clutter_actor_get_position (parent, &x, &y);
-      actor_x += x;
-      actor_y += y;
-
-      parent = clutter_actor_get_parent (parent);
-    }
+  clutter_actor_get_transformed_position (CLUTTER_ACTOR (self), &actor_x, &actor_y);
 
   area.x = (int)(0.5 + cursor_x + actor_x);
   area.y = (int)(0.5 + cursor_y + actor_y);
@@ -181,6 +163,13 @@ st_im_text_paint (ClutterActor *actor)
     update_im_cursor_location (self);
 }
 
+static gboolean
+st_im_text_get_paint_volume (ClutterActor       *self,
+                             ClutterPaintVolume *volume)
+{
+  return clutter_paint_volume_set_from_allocation (volume, self);
+}
+
 /* Returns a new reference to window */
 static GdkWindow *
 window_for_actor (ClutterActor *actor)
@@ -193,11 +182,11 @@ window_for_actor (ClutterActor *actor)
   stage = clutter_actor_get_stage (actor);
   xwindow = clutter_x11_get_stage_window ((ClutterStage *)stage);
 
-  window = gdk_window_lookup_for_display (display, xwindow);
+  window = gdk_x11_window_lookup_for_display (display, xwindow);
   if (window)
     g_object_ref (window);
   else
-    window = gdk_window_foreign_new_for_display (display, xwindow);
+    window = gdk_x11_window_foreign_new_for_display (display, xwindow);
 
   return window;
 }
@@ -231,31 +220,31 @@ key_is_modifier (guint16 keyval)
 
   switch (keyval)
     {
-    case GDK_Shift_L:
-    case GDK_Shift_R:
-    case GDK_Control_L:
-    case GDK_Control_R:
-    case GDK_Caps_Lock:
-    case GDK_Shift_Lock:
-    case GDK_Meta_L:
-    case GDK_Meta_R:
-    case GDK_Alt_L:
-    case GDK_Alt_R:
-    case GDK_Super_L:
-    case GDK_Super_R:
-    case GDK_Hyper_L:
-    case GDK_Hyper_R:
-    case GDK_ISO_Lock:
-    case GDK_ISO_Level2_Latch:
-    case GDK_ISO_Level3_Shift:
-    case GDK_ISO_Level3_Latch:
-    case GDK_ISO_Level3_Lock:
-    case GDK_ISO_Level5_Shift:
-    case GDK_ISO_Level5_Latch:
-    case GDK_ISO_Level5_Lock:
-    case GDK_ISO_Group_Shift:
-    case GDK_ISO_Group_Latch:
-    case GDK_ISO_Group_Lock:
+    case GDK_KEY_Shift_L:
+    case GDK_KEY_Shift_R:
+    case GDK_KEY_Control_L:
+    case GDK_KEY_Control_R:
+    case GDK_KEY_Caps_Lock:
+    case GDK_KEY_Shift_Lock:
+    case GDK_KEY_Meta_L:
+    case GDK_KEY_Meta_R:
+    case GDK_KEY_Alt_L:
+    case GDK_KEY_Alt_R:
+    case GDK_KEY_Super_L:
+    case GDK_KEY_Super_R:
+    case GDK_KEY_Hyper_L:
+    case GDK_KEY_Hyper_R:
+    case GDK_KEY_ISO_Lock:
+    case GDK_KEY_ISO_Level2_Latch:
+    case GDK_KEY_ISO_Level3_Shift:
+    case GDK_KEY_ISO_Level3_Latch:
+    case GDK_KEY_ISO_Level3_Lock:
+    case GDK_KEY_ISO_Level5_Shift:
+    case GDK_KEY_ISO_Level5_Latch:
+    case GDK_KEY_ISO_Level5_Lock:
+    case GDK_KEY_ISO_Group_Shift:
+    case GDK_KEY_ISO_Group_Latch:
+    case GDK_KEY_ISO_Group_Lock:
       return TRUE;
     default:
       return FALSE;
@@ -347,14 +336,6 @@ st_im_text_key_press_event (ClutterActor    *actor,
       gdk_event_free ((GdkEvent *)event_gdk);
     }
 
-  /* ClutterText:position isn't properly notified, so we have to
-   * check before/after to catch a keypress (like an arrow key)
-   * moving the cursor position, which should reset the IM context.
-   * (Resetting on notify::position would require a sentinel when
-   * committing text)
-   *
-   * http://bugzilla.openedhand.com/show_bug.cgi?id=1830
-   */
   old_position = clutter_text_get_cursor_position (clutter_text);
 
   if (!result &&
@@ -440,6 +421,7 @@ st_im_text_class_init (StIMTextClass *klass)
   object_class->dispose = st_im_text_dispose;
 
   actor_class->paint = st_im_text_paint;
+  actor_class->get_paint_volume = st_im_text_get_paint_volume;
   actor_class->realize = st_im_text_realize;
   actor_class->unrealize = st_im_text_unrealize;
 
