@@ -1115,6 +1115,7 @@ st_texture_cache_load (StTextureCache       *cache,
  * @cache:
  * @key: A cache key
  * @size: Size in pixels
+ * @policy: Cache policy
  * @request: (out): If no request is outstanding, one will be created and returned here
  * @texture: (out): A new texture, also added to the request
  *
@@ -1128,6 +1129,7 @@ static gboolean
 create_texture_and_ensure_request (StTextureCache        *cache,
                                    const char            *key,
                                    guint                  size,
+                                   StTextureCachePolicy   policy,
                                    AsyncTextureLoadData **request,
                                    ClutterActor         **texture)
 {
@@ -1154,7 +1156,8 @@ create_texture_and_ensure_request (StTextureCache        *cache,
     {
       /* Not cached and no pending request, create it */
       *request = g_new0 (AsyncTextureLoadData, 1);
-      g_hash_table_insert (cache->priv->outstanding_requests, g_strdup (key), *request);
+      if (policy != ST_TEXTURE_CACHE_POLICY_NONE)
+        g_hash_table_insert (cache->priv->outstanding_requests, g_strdup (key), *request);
     }
   else
    *request = pending;
@@ -1177,8 +1180,15 @@ load_gicon_with_colors (StTextureCache    *cache,
   char *key;
   GtkIconTheme *theme;
   GtkIconInfo *info;
+  StTextureCachePolicy policy;
 
   gicon_string = g_icon_to_string (icon);
+  /* A return value of NULL indicates that the icon can not be serialized,
+   * so don't have a unique identifier for it as a cache key, and thus can't
+   * be cached. If it is cachable, we hardcode a policy of FOREVER here for
+   * now; we should actually blow this away on icon theme changes probably */
+  policy = gicon_string != NULL ? ST_TEXTURE_CACHE_POLICY_FOREVER
+                                : ST_TEXTURE_CACHE_POLICY_NONE;
   if (colors)
     {
       /* This raises some doubts about the practice of using string keys */
@@ -1196,7 +1206,7 @@ load_gicon_with_colors (StTextureCache    *cache,
     }
   g_free (gicon_string);
 
-  if (create_texture_and_ensure_request (cache, key, size, &request, &texture))
+  if (create_texture_and_ensure_request (cache, key, size, policy, &request, &texture))
     {
       g_free (key);
       return texture;
@@ -1210,9 +1220,7 @@ load_gicon_with_colors (StTextureCache    *cache,
     {
       /* Transfer ownership of key */
       request->key = key;
-      /* hardcoded here for now; we should actually blow this away on
-       * icon theme changes probably */
-      request->policy = ST_TEXTURE_CACHE_POLICY_FOREVER;
+      request->policy = policy;
       request->icon = g_object_ref (icon);
       request->icon_info = info;
       request->width = request->height = size;
