@@ -189,20 +189,13 @@ corner_to_string (StCornerSpec *corner)
                           corner->border_width_2);
 }
 
-typedef struct {
-  StThemeNode *node;
-  StCornerSpec *corner;
-} LoadCornerData;
-
 static CoglHandle
 load_corner (StTextureCache  *cache,
              const char      *key,
              void            *datap,
              GError         **error)
 {
-  LoadCornerData *data = datap;
-
-  return create_corner_material (data->corner);
+  return create_corner_material ((StCornerSpec *) datap);
 }
 
 /* To match the CSS specification, we want the border to look like it was
@@ -347,15 +340,15 @@ st_theme_node_lookup_corner (StThemeNode    *node,
   char *key;
   StTextureCache *cache;
   StCornerSpec corner;
-  LoadCornerData data;
   guint radius[4];
 
-  if (node->border_radius[corner_id] == 0)
-    return COGL_INVALID_HANDLE;
-
   cache = st_texture_cache_get_default ();
 
   st_theme_node_reduce_border_radius (node, radius);
+
+  if (radius[corner_id] == 0)
+    return COGL_INVALID_HANDLE;
+
   corner.radius = radius[corner_id];
   corner.color = node->background_color;
   st_theme_node_get_corner_border_widths (node, corner_id,
@@ -388,10 +381,7 @@ st_theme_node_lookup_corner (StThemeNode    *node,
     return COGL_INVALID_HANDLE;
 
   key = corner_to_string (&corner);
-
-  data.node = node;
-  data.corner = &corner;
-  texture = st_texture_cache_load (cache, key, ST_TEXTURE_CACHE_POLICY_NONE, load_corner, &data, NULL);
+  texture = st_texture_cache_load (cache, key, ST_TEXTURE_CACHE_POLICY_NONE, load_corner, &corner, NULL);
   material = _st_create_texture_material (texture);
   cogl_handle_unref (texture);
 
@@ -601,8 +591,8 @@ create_cairo_pattern_of_background_image (StThemeNode *node,
           x_offset = - (node->alloc_width / 2. - scaled_width / 2.);
         }
 
-      cairo_matrix_init_translate (&matrix, x_offset, y_offset);
-      cairo_matrix_scale (&matrix, scale_factor, scale_factor);
+      cairo_matrix_init_scale (&matrix, scale_factor, scale_factor);
+      cairo_matrix_translate (&matrix, x_offset, y_offset);
 
       cairo_pattern_set_matrix (pattern, &matrix);
 
